import struct
import unittest

from common import get_context


class TestCase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.ctx = get_context()

    def test_1(self):
        rbo1 = self.ctx.renderbuffer((4, 4), dtype='f1')
        rbo2 = self.ctx.renderbuffer((4, 4))
        rbo3 = self.ctx.renderbuffer((4, 4))
        fbo1 = self.ctx.framebuffer(rbo1)
        fbo2 = self.ctx.framebuffer(rbo2)
        fbo3 = self.ctx.framebuffer(rbo3)
        fbo4 = self.ctx.framebuffer([rbo1, rbo2, rbo3])

        fbo1.clear(1.0, 0.0, 0.0, 1.0)
        fbo2.clear(0.0, 1.0, 0.0, 1.0)
        fbo3.clear(0.0, 0.0, 1.0, 1.0)

        buf = self.ctx.buffer(reserve=64)

        fbo4.read_into(buf, attachment=0, components=4, dtype='f1')
        self.assertEqual(buf.read(), b'\xff\x00\x00\xff' * 16)

        fbo4.read_into(buf, attachment=1, components=4, dtype='f1')
        self.assertEqual(buf.read(), b'\x00\xff\x00\xff' * 16)

        fbo4.read_into(buf, attachment=2, components=4, dtype='f1')
        self.assertEqual(buf.read(), b'\x00\x00\xff\xff' * 16)

    def test_2(self):
        pixels = b'\xFF\x00\x00\xFF\x00\xFF\x00\xFF\x00\x00\xFF\xFF\xFF\xFF\xFF\xFF'

        texture = self.ctx.texture((2, 2), 4)
        buf = self.ctx.buffer(pixels)
        texture.write(buf)

        self.assertEqual(texture.read(), pixels)

    def test_3(self):
        pixels = struct.pack(
            '4f4f4f4f',
            1.0, 0.0, 0.0, 1.0,
            0.0, 1.0, 0.0, 1.0,
            0.0, 0.0, 1.0, 1.0,
            1.0, 1.0, 1.0, 1.0,
        )

        buf = self.ctx.buffer(pixels)
        texture = self.ctx.texture((2, 2), 4, dtype='f4')
        texture.write(buf)

        self.assertEqual(texture.read(), pixels)

    def test_4(self):
        pixels = b'\xFF\x00\x00\xFF\x00\xFF\x00\xFF\x00\x00\xFF\xFF\xFF\xFF\xFF\xFF'

        texture = self.ctx.texture((2, 2), 4, pixels)
        buf = self.ctx.buffer(reserve=len(pixels))
        texture.read_into(buf)

        self.assertEqual(buf.read(), pixels)

    def test_5(self):
        pixels = struct.pack(
            '4f4f4f4f',
            1.0, 0.0, 0.0, 1.0,
            0.0, 1.0, 0.0, 1.0,
            0.0, 0.0, 1.0, 1.0,
            1.0, 1.0, 1.0, 1.0,
        )

        texture = self.ctx.texture((2, 2), 4, pixels, dtype='f4')
        buf = self.ctx.buffer(reserve=len(pixels))
        texture.read_into(buf)

        self.assertEqual(buf.read(), pixels)


if __name__ == '__main__':
    unittest.main()



import unittest

import moderngl

from common import get_context


class TestCase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.ctx = get_context()

        cls.vertex_shader = '''
            #version 330
            uniform mat4 Mvp;
            in vec4 vertex;
            uniform Block {
                mat4 Transform;
            };
            void main() {
                gl_Position = Mvp * Transform * vertex;
            }
        '''

        cls.geometry_shader = '''
            #version 330
            layout(points) in;
            uniform vec4 Position;
            layout(line_strip, max_vertices = 2) out;
            out vec4 color;
            void main() {
                gl_Position = gl_in[0].gl_Position + Position;
                EmitVertex();
                gl_Position = gl_in[0].gl_Position - Position;
                EmitVertex();
                EndPrimitive();
            }
        '''

        cls.fragment_shader = '''
            #version 330
            uniform vec4 Color;
            uniform Lights {
                uniform vec4 Light[1];
            };
            out vec4 color;
            void main() {
                color = Color * Light[0];
            }
        '''

    def test_program_uniforms_1(self):
        program = self.ctx.program(vertex_shader=self.vertex_shader)

        self.assertIn('Mvp', program)
        self.assertIn('Block', program)
        self.assertNotIn('Transform', program)

    def test_program_uniforms_2(self):
        program = self.ctx.program(
            vertex_shader=self.vertex_shader,
            geometry_shader=self.geometry_shader,
            fragment_shader=self.fragment_shader,
        )

        self.assertIn('Mvp', program)
        self.assertIn('Position', program)
        self.assertIn('Color', program)

    def test_program_uniform_blocks(self):
        program = self.ctx.program(
            vertex_shader=self.vertex_shader,
            fragment_shader=self.fragment_shader,
        )

        self.assertIn('Block', program)
        self.assertIn('Lights', program)
        self.assertNotIn('Transform', program)
        self.assertNotIn('Light', program)

    def test_program_geometry_primitives_1(self):
        program = self.ctx.program(vertex_shader=self.vertex_shader)

        self.assertIsNone(program.geometry_input)
        self.assertIsNone(program.geometry_output)
        self.assertEqual(program.geometry_vertices, 0)

    def test_program_geometry_primitives_2(self):
        program = self.ctx.program(
            vertex_shader=self.vertex_shader,
            geometry_shader=self.geometry_shader,
        )
        self.assertEqual(program.geometry_vertices, 2)


if __name__ == '__main__':
    unittest.main()



import unittest

from common import get_context


class TestCase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.ctx = get_context()

    def test_1(self):
        buf = self.ctx.buffer(b'abc')
        self.assertEqual(buf.read(), b'abc')
        buf.write(b'xyz')
        self.assertEqual(buf.read(), b'xyz')
        self.assertEqual(buf.read(offset=1), b'yz')

    def test_2(self):
        buf = self.ctx.buffer(reserve=10)
        self.assertEqual(buf.size, 10)
        buf.write(b'12345')
        buf.write(b'abcde', offset=5)
        self.assertEqual(buf.read(5), b'12345')
        self.assertEqual(buf.read(5, offset=5), b'abcde')

    def test_3(self):
        buf = self.ctx.buffer(reserve='1KB')
        self.assertEqual(buf.size, 1024)
        buf.clear(chunk=b'u')
        self.assertEqual(buf.read(), b'u' * 1024)
        buf.clear(size=10, offset=10, chunk=b'ab')
        self.assertEqual(buf.read(10, offset=0), b'uuuuuuuuuu')
        self.assertEqual(buf.read(10, offset=10), b'ababababab')
        self.assertEqual(buf.read(10, offset=20), b'uuuuuuuuuu')

    def test_4(self):
        buf = self.ctx.buffer(reserve=10)
        buf.write_chunks(b'00000', 0, 2, 5)
        buf.write_chunks(b'11111', 1, 2, 5)
        self.assertEqual(buf.read(10, offset=0), b'0101010101')

    def test_5(self):
        buf = self.ctx.buffer(b'123456789')
        buf.write_chunks(b'AABBCC', 0, 3, 3)
        self.assertEqual(buf.read(), b'AA3BB6CC9')
        self.assertEqual(buf.read_chunks(2, 0, 3, 3), b'AABBCC')

    def test_6(self):
        buf = self.ctx.buffer(b'123456789')
        buf.write_chunks(b'XYZ', -1, -3, 3)
        self.assertEqual(buf.read(), b'12Z45Y78X')
        self.assertEqual(buf.read_chunks(1, -1, -3, 3), b'XYZ')

    def test_7(self):
        buf = self.ctx.buffer(b'123456789')
        self.assertEqual(buf.read_chunks(3, 0, 3, 3), b'123456789')

    def test_8(self):
        buf1 = self.ctx.buffer(b'abc', dynamic=True)
        buf2 = self.ctx.buffer(b'abc', dynamic=False)
        self.assertTrue(buf1.dynamic)
        self.assertFalse(buf2.dynamic)


if __name__ == '__main__':
    unittest.main()



import moderngl

_static = {
    'context': None,
}


def get_context() -> moderngl.Context:
    ctx = _static.get('context')

    if ctx is None:
        ctx = moderngl.create_standalone_context(size=(100, 100))
        _static['context'] = ctx

    return ctx



import unittest

from common import get_context
import moderngl


class TestCase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.ctx = get_context()

    def test_program(self):
        program = self.ctx.program(
            vertex_shader='''
                #version 330

                uniform vec2 pos;
                uniform float scale;

                in vec2 vert;
                out vec2 v_vert;

                void main() {
                    gl_Position = vec4(pos + vert * scale, 0.0, 1.0);
                    v_vert = vert;
                }
            ''',
            fragment_shader='''
                #version 330

                in vec2 v_vert;
                out vec4 color;

                void main() {
                    color = vec4(v_vert, 0.0, 1.0);
                }
            ''',
        )

        self.assertIn('vert', program)
        self.assertIn('pos', program)
        self.assertIn('scale', program)

        self.assertIsInstance(program['vert'], moderngl.Attribute)
        self.assertIsInstance(program['pos'], moderngl.Uniform)
        self.assertIsInstance(program['scale'], moderngl.Uniform)


if __name__ == '__main__':
    unittest.main()



import inspect
import os
import re
import unittest

import moderngl


class TestCase(unittest.TestCase):

    def validate(self, filename, classname, ignore):
        root = os.path.dirname(os.path.dirname(__file__))
        with open(os.path.normpath(os.path.join(root, 'docs', 'reference', filename))) as f:
            docs = f.read()
        methods = re.findall(r'^\.\. automethod:: ([^\(\n]+)([^\n]+)', docs, flags=re.M)
        attributes = re.findall(r'^\.\. autoattribute:: ([^\n]+)', docs, flags=re.M)
        documented = set(filter(lambda x: x.startswith(classname), [a for a, b in methods] + attributes))
        implemented = set(classname + '.' + x for x in dir(getattr(moderngl, classname)) if not x.startswith('_'))
        ignored = set(classname + '.' + x for x in ignore)
        self.assertSetEqual(implemented - documented - ignored, set(), msg='Implemented but not Documented')
        self.assertSetEqual(documented - implemented, set(), msg='Documented but not Implemented')

        for method, docsig in methods:
            classname, methodname = method.split('.')
            sig = str(inspect.signature(getattr(getattr(moderngl, classname), methodname)))
            sig = sig.replace('self, ', '').replace('moderngl.', '').replace('typing.', '').replace(' -> None', '')
            sig = sig.replace('(self)', '()').replace(', *,', ',').replace('(*, ', '(')
            sig = re.sub(r'-> \'(\w+)\'', r'-> \1', sig)
            self.assertEqual(docsig, sig, msg=filename + '::' + method)

    def test_context_docs(self):
        self.validate('context.rst', 'Context', ['release', 'mglo', 'extra'])

    def test_program_docs(self):
        self.validate('program.rst', 'Program', ['release', 'mglo', 'glo', 'ctx', 'extra'])

    def test_vertex_array_docs(self):
        self.validate('vertex_array.rst', 'VertexArray', ['release', 'mglo', 'glo', 'ctx', 'extra'])

    def test_buffer_docs(self):
        self.validate('buffer.rst', 'Buffer', ['release', 'mglo', 'glo', 'ctx', 'extra'])

    def test_texture_docs(self):
        self.validate('texture.rst', 'Texture', ['release', 'mglo', 'glo', 'ctx', 'extra'])

    def test_texture3d_docs(self):
        self.validate('texture3d.rst', 'Texture3D', ['release', 'mglo', 'glo', 'ctx', 'extra'])

    def test_texture_cube_docs(self):
        self.validate('texture_cube.rst', 'TextureCube', ['release', 'mglo', 'glo', 'ctx', 'extra'])

    def test_framebuffer_docs(self):
        self.validate('framebuffer.rst', 'Framebuffer', ['release', 'mglo', 'glo', 'ctx', 'extra'])

    def test_renderbuffer_docs(self):
        self.validate('renderbuffer.rst', 'Renderbuffer', ['release', 'mglo', 'glo', 'ctx', 'extra'])

    def test_query_docs(self):
        self.validate('query.rst', 'Query', ['mglo', 'ctx', 'extra'])

    def test_scope_docs(self):
        self.validate('scope.rst', 'Scope', ['mglo', 'ctx', 'extra'])

    def test_compute_shader_docs(self):
        self.validate('compute_shader.rst', 'ComputeShader', ['release', 'mglo', 'glo', 'ctx', 'extra'])

    def test_subroutine_docs(self):
        self.validate('subroutine.rst', 'Subroutine', ['mglo', 'ctx', 'extra'])

    def test_uniform_docs(self):
        self.validate('uniform.rst', 'Uniform', ['mglo', 'extra'])

    def test_uniform_block_docs(self):
        self.validate('uniform_block.rst', 'UniformBlock', ['mglo', 'extra'])

    def test_varying_docs(self):
        self.validate('varying.rst', 'Varying', ['mglo', 'extra'])

    def test_conditional_render_docs(self):
        self.validate('conditional_render.rst', 'ConditionalRender', ['mglo'])


if __name__ == '__main__':
    unittest.main()



import unittest

from common import get_context


class TestCase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.ctx = get_context()

    def test_1(self):
        fbo = self.ctx.framebuffer(self.ctx.renderbuffer((4, 4)))

        fbo.clear(0.0, 0.0, 0.0, 0.0)

        self.assertEqual(fbo.read(components=4), b'\x00\x00\x00\x00' * 16)

        fbo.color_mask = (True, False, True, False)
        fbo.clear(0x19 / 255, 0x33 / 255, 0x4c / 255, 0x66 / 255)

        self.assertEqual(fbo.read(components=4), b'\x19\x00\x4c\x00' * 16)

        fbo.color_mask = (False, True, False, True)
        fbo.clear(0x7f / 255, 0x99 / 255, 0xb2 / 255, 0xcc / 255)

        self.assertEqual(fbo.read(components=4), b'\x19\x99\x4c\xcc' * 16)

    def test_2(self):
        fbo = self.ctx.framebuffer([
            self.ctx.renderbuffer((4, 4)),
            self.ctx.renderbuffer((4, 4)),
        ])

        fbo.clear(1.0, 1.0, 1.0, 1.0)

        self.assertEqual(fbo.read(components=4, attachment=0), b'\xff\xff\xff\xff' * 16)
        self.assertEqual(fbo.read(components=4, attachment=1), b'\xff\xff\xff\xff' * 16)

        fbo.color_mask = (
            (True, False, True, False),
            (False, True, False, True),
        )

        fbo.clear(0x19 / 255, 0x33 / 255, 0x4c / 255, 0x66 / 255)

        self.assertEqual(fbo.read(components=4, attachment=0), b'\x19\xff\x4c\xff' * 16)
        self.assertEqual(fbo.read(components=4, attachment=1), b'\xff\x33\xff\x66' * 16)

        fbo.color_mask = (
            (False, True, False, True),
            (True, False, True, False),
        )

        fbo.clear(0x7f / 255, 0x99 / 255, 0xb2 / 255, 0xcc / 255)

        self.assertEqual(fbo.read(components=4, attachment=0), b'\x19\x99\x4c\xcc' * 16)
        self.assertEqual(fbo.read(components=4, attachment=1), b'\x7f\x33\xb2\x66' * 16)


if __name__ == '__main__':
    unittest.main()



import struct
import unittest

import moderngl

from common import get_context


class TestCase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.ctx = get_context()

    def test_vao_attribs(self):

        program = self.ctx.program(
            vertex_shader='''
                #version 330

                layout(location = 0) in vec4 in_vert;
                out vec4 out_vert;

                void main() {
                    out_vert = in_vert;
                }
            ''',
            varyings=['out_vert']
        )

        vbo = self.ctx.buffer(struct.pack('12f', 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12))
        res = self.ctx.buffer(reserve=struct.calcsize('8f'))

        vao = self.ctx.simple_vertex_array(program, vbo, 'in_vert')
        vao.transform(res, moderngl.POINTS, 2)

        for a, b in zip(struct.unpack('8f', res.read()), (1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0)):
            self.assertAlmostEqual(a, b)

        vao.bind(0, 'f', vbo, '4f', offset=4, stride=0, divisor=0)
        vao.transform(res, moderngl.POINTS, 2)

        for a, b in zip(struct.unpack('8f', res.read()), (2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0)):
            self.assertAlmostEqual(a, b)

        vao.bind(0, 'f', vbo, '4f', offset=8, stride=20, divisor=0)
        vao.transform(res, moderngl.POINTS, 2)

        for a, b in zip(struct.unpack('8f', res.read()), (3.0, 4.0, 5.0, 6.0, 8.0, 9.0, 10.0, 11.0)):
            self.assertAlmostEqual(a, b)

        vao.bind(0, 'f', vbo, '4f', offset=12, stride=0, divisor=1)
        vao.transform(res, moderngl.POINTS, 2)

        for a, b in zip(struct.unpack('8f', res.read()), (4.0, 5.0, 6.0, 7.0, 4.0, 5.0, 6.0, 7.0)):
            self.assertAlmostEqual(a, b)


if __name__ == '__main__':
    unittest.main()



import struct
import unittest

import moderngl

from common import get_context


class TestCase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.ctx = get_context()

        prog = cls.ctx.program(
            vertex_shader='''
                #version 330
                in vec2 vert;
                out vec2 text;
                void main() {
                    gl_Position = vec4(vert * 2.0 - 1.0, 0.0, 1.0);
                    text = vert;
                }
            ''',
            fragment_shader='''
                #version 330
                uniform sampler2D Texture;
                in vec2 text;
                out vec4 color;
                void main() {
                    color = texture(Texture, text);
                }
            ''',
        )

        vbo = cls.ctx.buffer(struct.pack('8f', 0, 0, 0, 1, 1, 0, 1, 1))
        cls.vao = cls.ctx.simple_vertex_array(prog, vbo, 'vert')

    def test_1(self):
        fbo = self.ctx.framebuffer(self.ctx.renderbuffer((16, 16)))
        pixels = struct.pack('16B', 255, 0, 0, 255, 0, 255, 0, 255, 0, 0, 255, 255, 0, 0, 0, 255)
        texture = self.ctx.texture((2, 2), 4, pixels)

        fbo.use()
        texture.use()

        texture.filter = (moderngl.LINEAR, moderngl.LINEAR)
        self.vao.render(moderngl.TRIANGLE_STRIP)

        r, g, b = struct.unpack('3B', fbo.read((8, 8, 1, 1)))
        self.assertTrue(abs(r - 49) < 8 and abs(g - 63) < 8 and abs(b - 63) < 8)

        texture.filter = (moderngl.NEAREST, moderngl.NEAREST)
        self.vao.render(moderngl.TRIANGLE_STRIP)

        r, g, b = struct.unpack('3B', fbo.read((6, 6, 1, 1)))
        self.assertTrue(abs(r - 255) < 8 and abs(g - 0) < 8 and abs(b - 0) < 8)

        r, g, b = struct.unpack('3B', fbo.read((9, 6, 1, 1)))
        self.assertTrue(abs(r - 0) < 8 and abs(g - 255) < 8 and abs(b - 0) < 8)

        r, g, b = struct.unpack('3B', fbo.read((6, 9, 1, 1)))
        self.assertTrue(abs(r - 0) < 8 and abs(g - 0) < 8 and abs(b - 255) < 8)

        r, g, b = struct.unpack('3B', fbo.read((9, 9, 1, 1)))
        self.assertTrue(abs(r - 0) < 8 and abs(g - 0) < 8 and abs(b - 0) < 8)

    def test_2(self):
        fbo = self.ctx.framebuffer(self.ctx.renderbuffer((16, 16)))
        pixels1 = struct.pack('16B', 255, 0, 0, 255, 0, 255, 0, 255, 255, 0, 0, 255, 0, 255, 0, 255)
        pixels2 = struct.pack('16B', 20, 30, 40, 255, 80, 70, 60, 255, 20, 30, 40, 255, 80, 70, 60, 255)
        texture = self.ctx.texture((2, 2), 4, pixels1)

        fbo.use()
        texture.use()

        texture.filter = (moderngl.NEAREST, moderngl.NEAREST)
        self.vao.render(moderngl.TRIANGLE_STRIP)

        r, g, b = struct.unpack('3B', fbo.read((5, 7, 1, 1)))
        self.assertTrue(abs(r - 255) < 8 and abs(g - 0) < 8 and abs(b - 0) < 8)

        r, g, b = struct.unpack('3B', fbo.read((10, 7, 1, 1)))
        self.assertTrue(abs(r - 0) < 8 and abs(g - 255) < 8 and abs(b - 0) < 8)

        texture.write(pixels2)

        texture.filter = (moderngl.NEAREST, moderngl.NEAREST)
        self.vao.render(moderngl.TRIANGLE_STRIP)

        r, g, b = struct.unpack('3B', fbo.read((5, 7, 1, 1)))
        self.assertTrue(abs(r - 20) < 8 and abs(g - 30) < 8 and abs(b - 40) < 8)

        r, g, b = struct.unpack('3B', fbo.read((10, 7, 1, 1)))
        self.assertTrue(abs(r - 80) < 8 and abs(g - 70) < 8 and abs(b - 60) < 8)

    def test_3(self):
        fbo = self.ctx.framebuffer(self.ctx.renderbuffer((16, 16)))
        pixels1 = struct.pack('16B', 255, 0, 0, 255, 0, 255, 0, 255, 255, 0, 0, 255, 0, 255, 0, 255)
        pixels2 = struct.pack('16B', 20, 30, 40, 255, 80, 70, 60, 255, 20, 30, 40, 255, 80, 70, 60, 255)
        texture = self.ctx.texture((2, 2), 4)

        pbo1 = self.ctx.buffer(pixels1)
        pbo2 = self.ctx.buffer(pixels2)

        fbo.use()
        texture.use()

        texture.write(pbo1)
        texture.filter = (moderngl.NEAREST, moderngl.NEAREST)
        self.vao.render(moderngl.TRIANGLE_STRIP)

        r, g, b = struct.unpack('3B', fbo.read((5, 7, 1, 1)))
        self.assertTrue(abs(r - 255) < 8 and abs(g - 0) < 8 and abs(b - 0) < 8)

        r, g, b = struct.unpack('3B', fbo.read((10, 7, 1, 1)))
        self.assertTrue(abs(r - 0) < 8 and abs(g - 255) < 8 and abs(b - 0) < 8)

        texture.write(pbo2)
        texture.filter = (moderngl.NEAREST, moderngl.NEAREST)
        self.vao.render(moderngl.TRIANGLE_STRIP)

        r, g, b = struct.unpack('3B', fbo.read((5, 7, 1, 1)))
        self.assertTrue(abs(r - 20) < 8 and abs(g - 30) < 8 and abs(b - 40) < 8)

        r, g, b = struct.unpack('3B', fbo.read((10, 7, 1, 1)))
        self.assertTrue(abs(r - 80) < 8 and abs(g - 70) < 8 and abs(b - 60) < 8)

    def test_4(self):
        pixels = struct.pack('16B', 255, 0, 0, 255, 0, 255, 0, 255, 255, 0, 0, 255, 0, 255, 0, 255)
        texture = self.ctx.texture((2, 2), 4, pixels)
        self.assertEqual(texture.read(), pixels)

    def test_5(self):
        pixels = struct.pack('16B', 255, 0, 0, 255, 0, 255, 0, 255, 255, 0, 0, 255, 0, 255, 0, 255)
        texture = self.ctx.texture((2, 2), 4, pixels)
        buf = bytearray(len(pixels))
        texture.read_into(buf)

        self.assertEqual(bytes(buf), pixels)

    def test_6(self):
        pixels = struct.pack('16B', 255, 0, 0, 255, 0, 255, 0, 255, 255, 0, 0, 255, 0, 255, 0, 255)
        texture = self.ctx.texture((2, 2), 4, pixels)
        pbo = self.ctx.buffer(reserve=len(pixels))
        texture.read_into(pbo)

        self.assertEqual(pbo.read(), pixels)


if __name__ == '__main__':
    unittest.main()



import unittest

from common import get_context


class TestCase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.ctx = get_context()

    def test_1(self):
        buf = self.ctx.buffer(b'abc')

        with self.assertRaises(Exception):
            buf.write(b'1234')

        with self.assertRaises(Exception):
            buf.write(b'u', offset=-1)

        with self.assertRaises(Exception):
            buf.write(b'abc', offset=1)

    def test_2(self):
        buf = self.ctx.buffer(b'123456789')

        with self.assertRaises(Exception):
            buf.write_chunks(b'12345', 0, 1, 4)

        with self.assertRaises(Exception):
            buf.write_chunks(b'12345', 0, 1, 6)

        with self.assertRaises(Exception):
            buf.write_chunks(b'yyyyyn', 4, 1, 6)

        with self.assertRaises(Exception):
            buf.write_chunks(b'yyyyyn', 4, -1, 6)

        with self.assertRaises(Exception):
            buf.write_chunks(b'yn', -1, 2, 1)

    def test_3(self):
        buf = self.ctx.buffer(b'123456789')

        with self.assertRaises(Exception):
            buf.write_chunks(b'yyyyyn', 2, 3, 3)

        with self.assertRaises(Exception):
            buf.write_chunks(b'ynyyyy', -1, -3, 3)

        with self.assertRaises(Exception):
            buf.write_chunks(b'yyyyny', -4, -3, 3)

    def test_4(self):
        buf = self.ctx.buffer(b'123456789')

        with self.assertRaises(Exception):
            buf.write_chunks(b'yyynyy', 0, 2, 2)


if __name__ == '__main__':
    unittest.main()



import unittest

from common import get_context


class TestCase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.ctx = get_context()

    def test_1(self):
        buf1 = self.ctx.buffer(b'abc')
        buf2 = self.ctx.buffer(reserve=3)
        self.ctx.copy_buffer(buf2, buf1)
        self.assertEqual(buf2.read(), b'abc')

    def test_2(self):
        buf1 = self.ctx.buffer(b'abcxyz123')
        buf2 = self.ctx.buffer(reserve=12)
        self.ctx.copy_buffer(buf2, buf1, 3, read_offset=3, write_offset=0)
        self.ctx.copy_buffer(buf2, buf1, 3, read_offset=0, write_offset=3)
        self.ctx.copy_buffer(buf2, buf1, 3, read_offset=6, write_offset=6)
        self.ctx.copy_buffer(buf2, buf1, 3, read_offset=3, write_offset=9)
        self.assertEqual(buf2.read(), b'xyzabc123xyz')


if __name__ == '__main__':
    unittest.main()



import struct
import unittest

from common import get_context


class TestCase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.ctx = get_context()

    def test_1(self):
        if self.ctx.version_code < 400:
            self.skipTest('OpenGL 4.0 is not supported')

        vbo1 = self.ctx.buffer(struct.pack('4f', 0.0, 0.0, 0.0, 0.0))
        vbo2 = self.ctx.buffer(struct.pack('4f', 0.0, 0.0, 0.0, 0.0))

        prog = self.ctx.program(
            vertex_shader='''
                #version 400

                in vec4 vert;
                out vec4 color;

                subroutine vec4 color_t();

                subroutine(color_t)
                vec4 ColorRed() {
                    return vec4(1, 0, 0, 1);
                }

                subroutine(color_t)
                vec4 ColorBlue() {
                    return vec4(0, 0.5, 1, 1);
                }

                subroutine(color_t)
                vec4 ColorYellow() {
                    return vec4(1, 1, 0, 1);
                }

                subroutine uniform color_t Color;

                void main() {
                    color = vert + Color();
                }
            ''',
            varyings=['color']
        )
        vao = self.ctx.simple_vertex_array(prog, vbo1, 'vert')

        vao.subroutines = [prog['ColorRed'].index]
        vao.transform(vbo2)

        x, y, z, w = struct.unpack('4f', vbo2.read())
        self.assertAlmostEqual(x, 1.0)
        self.assertAlmostEqual(y, 0.0)
        self.assertAlmostEqual(z, 0.0)
        self.assertAlmostEqual(w, 1.0)

        vao.subroutines = [prog['ColorBlue'].index]
        vao.transform(vbo2)

        x, y, z, w = struct.unpack('4f', vbo2.read())
        self.assertAlmostEqual(x, 0.0)
        self.assertAlmostEqual(y, 0.5)
        self.assertAlmostEqual(z, 1.0)
        self.assertAlmostEqual(w, 1.0)

        vao.subroutines = [prog['ColorYellow'].index]
        vao.transform(vbo2)

        x, y, z, w = struct.unpack('4f', vbo2.read())
        self.assertAlmostEqual(x, 1.0)
        self.assertAlmostEqual(y, 1.0)
        self.assertAlmostEqual(z, 0.0)
        self.assertAlmostEqual(w, 1.0)


if __name__ == '__main__':
    unittest.main()



import unittest

import moderngl

from common import get_context


class TestCase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.ctx = get_context()
        cls.max_samples = cls.ctx.info['GL_MAX_SAMPLES']

    def test_framebuffer_color_attachment(self):
        rbo = self.ctx.renderbuffer((16, 16))
        self.ctx.framebuffer(rbo)

    def test_framebuffer_get_color_attachments(self):
        rbo1 = self.ctx.renderbuffer((16, 16))
        rbo2 = self.ctx.renderbuffer((16, 16))
        rbo3 = self.ctx.renderbuffer((16, 16))

        fbo1 = self.ctx.framebuffer(rbo1)
        fbo2 = self.ctx.framebuffer([rbo2, rbo1])
        fbo3 = self.ctx.framebuffer([rbo1, rbo2, rbo3])

        self.assertEqual(len(fbo1.color_attachments), 1)
        self.assertEqual(len(fbo2.color_attachments), 2)
        self.assertEqual(len(fbo3.color_attachments), 3)

        self.assertIsInstance(fbo1.color_attachments[0], moderngl.Renderbuffer)
        self.assertIsInstance(fbo2.color_attachments[0], moderngl.Renderbuffer)
        self.assertIsInstance(fbo2.color_attachments[1], moderngl.Renderbuffer)
        self.assertIsInstance(fbo3.color_attachments[0], moderngl.Renderbuffer)
        self.assertIsInstance(fbo3.color_attachments[1], moderngl.Renderbuffer)
        self.assertIsInstance(fbo3.color_attachments[2], moderngl.Renderbuffer)

        self.assertIn(rbo1, fbo1.color_attachments)
        self.assertIn(rbo2, fbo2.color_attachments)
        self.assertIn(rbo2, fbo3.color_attachments)

        self.assertNotIn(rbo2, fbo1.color_attachments)
        self.assertNotIn(rbo3, fbo1.color_attachments)
        self.assertNotIn(rbo3, fbo2.color_attachments)

    def test_framebuffer_get_depth_attachment(self):
        rbo1 = self.ctx.renderbuffer((16, 16))
        rbo2 = self.ctx.depth_renderbuffer((16, 16))

        fbo1 = self.ctx.framebuffer(rbo1)
        fbo2 = self.ctx.framebuffer(rbo1, rbo2)

        self.assertIsNone(fbo1.depth_attachment)
        self.assertIsInstance(fbo2.depth_attachment, moderngl.Renderbuffer)

        self.assertNotEqual(fbo1.depth_attachment, rbo2)
        self.assertEqual(fbo2.depth_attachment, rbo2)

    def test_framebuffer_color_mask(self):
        fbo = self.ctx.framebuffer(self.ctx.renderbuffer((16, 16)))
        self.assertEqual(fbo.color_mask, (True, True, True, True))

    def test_framebuffer_single_channel_color_mask(self):
        fbo = self.ctx.framebuffer(self.ctx.renderbuffer((16, 16), components=1))
        self.assertEqual(fbo.color_mask, (True, False, False, False))

    def test_framebuffer_mixed_channels_color_mask(self):
        fbo = self.ctx.framebuffer([
            self.ctx.renderbuffer((16, 16), components=1),
            self.ctx.renderbuffer((16, 16), components=2),
            self.ctx.renderbuffer((16, 16), components=3),
            self.ctx.renderbuffer((16, 16), components=4),
        ])
        expected = (
            (True, False, False, False),
            (True, True, False, False),
            (True, True, True, False),
            (True, True, True, True),
        )
        self.assertEqual(fbo.color_mask, expected)

    def test_framebuffer_depth_mask_1(self):
        fbo = self.ctx.framebuffer(
            self.ctx.renderbuffer((16, 16)),
            self.ctx.depth_renderbuffer((16, 16)),
        )
        self.assertEqual(fbo.depth_mask, True)

    def test_framebuffer_depth_mask_2(self):
        fbo = self.ctx.framebuffer(
            self.ctx.renderbuffer((16, 16))
        )
        self.assertEqual(fbo.depth_mask, False)

    def test_framebuffer_color_attachments(self):
        rbo1 = self.ctx.renderbuffer((16, 16))
        rbo2 = self.ctx.renderbuffer((16, 16))
        rbo3 = self.ctx.renderbuffer((16, 16))
        self.ctx.framebuffer([rbo1, rbo2, rbo3])

    def test_framebuffer_multiple_color_masks(self):
        fbo = self.ctx.framebuffer([
            self.ctx.renderbuffer((16, 16)),
            self.ctx.renderbuffer((16, 16)),
        ])
        expected = ((True, True, True, True), (True, True, True, True))
        self.assertEqual(fbo.color_mask, expected)

    def test_framebuffer_size_mismatch(self):
        with self.assertRaisesRegex(moderngl.Error, 'size'):
            rbo1 = self.ctx.renderbuffer((16, 16))
            rbo2 = self.ctx.depth_renderbuffer((32, 32))
            self.ctx.framebuffer(rbo1, rbo2)

    def test_framebuffer_color_attachments_size_mismatch(self):
        with self.assertRaisesRegex(moderngl.Error, 'size'):
            rbo1 = self.ctx.renderbuffer((16, 16))
            rbo2 = self.ctx.renderbuffer((32, 32))
            self.ctx.framebuffer([rbo1, rbo2])

    def test_depth_framebuffer(self):
        rbo1 = self.ctx.renderbuffer((16, 16))
        rbo2 = self.ctx.depth_renderbuffer((16, 16))
        self.ctx.framebuffer(rbo1, rbo2)

    def test_framebuffer_multisample(self):
        if self.max_samples < 2:
            self.skipTest('multisampling is not supported')

        rbo1 = self.ctx.renderbuffer((16, 16), samples=2)
        self.ctx.framebuffer(rbo1)

    def test_depth_framebuffer_multisample(self):
        if self.max_samples < 2:
            self.skipTest('multisampling is not supported')

        rbo1 = self.ctx.renderbuffer((16, 16), samples=2)
        rbo2 = self.ctx.depth_renderbuffer((16, 16), samples=2)
        self.ctx.framebuffer(rbo1, rbo2)

    def test_framebuffer_multisample_sample_mismatch(self):
        if self.max_samples < 2:
            self.skipTest('multisampling is not supported')

        with self.assertRaisesRegex(moderngl.Error, 'sample'):
            rbo1 = self.ctx.renderbuffer((16, 16))
            rbo2 = self.ctx.depth_renderbuffer((16, 16), samples=2)
            self.ctx.framebuffer(rbo1, rbo2)

    def test_empty_framebuffer(self):
        with self.assertRaisesRegex(moderngl.Error, 'empty'):
            self.ctx.framebuffer([])

    def test_framebuffer_having_depth_in_colors(self):
        with self.assertRaisesRegex(moderngl.Error, r'(color|depth)'):
            self.ctx.framebuffer(self.ctx.depth_renderbuffer((16, 16)))

    def test_framebuffer_having_color_in_depth(self):
        with self.assertRaisesRegex(moderngl.Error, r'(color|depth)'):
            self.ctx.framebuffer(
                self.ctx.renderbuffer((16, 16)),
                self.ctx.renderbuffer((16, 16)),
            )


if __name__ == '__main__':
    unittest.main()



import platform
import sys

from setuptools import Extension, setup

# pylint: disable=C0103, W0212

if sys.version_info < (3, 0):
    raise Exception('Python 2 is not supported!')

PLATFORMS = {'windows', 'linux', 'darwin', 'cygwin'}

target = platform.system().lower()

for known in PLATFORMS:
    if target.startswith(known):
        target = known

if target not in PLATFORMS:
    target = 'linux'

if target in ['linux', 'cygwin']:
    from distutils import sysconfig
    cvars = sysconfig.get_config_vars()

    if 'OPT' in cvars:
        sysconfig._config_vars['OPT'] = cvars['OPT'].replace('-Wstrict-prototypes', '')
        sysconfig._config_vars['OPT'] = cvars['OPT'].replace('-Wimplicit-function-declaration', '')

    if 'CFLAGS' in cvars:
        sysconfig._config_vars['CFLAGS'] = cvars['CFLAGS'].replace('-Wstrict-prototypes', '')
        sysconfig._config_vars['CFLAGS'] = cvars['CFLAGS'].replace('-Wimplicit-function-declaration', '')

install_requires = []

if sys.version_info < (3, 5):
    install_requires.append('typing')

libraries = {
    'windows': ['gdi32', 'opengl32', 'user32'],
    'linux': ['GL', 'dl', 'X11'],
    'cygwin': ['GL', 'X11'],
    'darwin': [],
}

extra_compile_args = {
    'windows': [],
    'linux': [],
    'cygwin': [],
    'darwin': ['-Wno-deprecated-declarations'],
}

extra_linker_args = {
    'windows': [],
    'linux': [],
    'cygwin': [],
    'darwin': ['-framework', 'OpenGL', '-Wno-deprecated'],
}

mgl = Extension(
    name='moderngl.mgl',
    include_dirs=['src'],
    define_macros=[],
    libraries=libraries[target],
    extra_compile_args=extra_compile_args[target],
    extra_link_args=extra_linker_args[target],
    sources=[
        'src/Attribute.cpp',
        'src/Buffer.cpp',
        'src/BufferFormat.cpp',
        'src/ComputeShader.cpp',
        'src/Context.cpp',
        'src/DataType.cpp',
        'src/Error.cpp',
        'src/Framebuffer.cpp',
        'src/GLContext.cpp',
        'src/GLMethods.cpp',
        'src/InvalidObject.cpp',
        'src/ModernGL.cpp',
        'src/Program.cpp',
        'src/Query.cpp',
        'src/Renderbuffer.cpp',
        'src/Scope.cpp',
        'src/Texture.cpp',
        'src/Texture3D.cpp',
        'src/TextureArray.cpp',
        'src/TextureCube.cpp',
        'src/Uniform.cpp',
        'src/UniformBlock.cpp',
        'src/UniformGetters.cpp',
        'src/UniformSetters.cpp',
        'src/VertexArray.cpp',
    ],
)

short_description = 'ModernGL: High performance rendering for Python 3'

long_description = '''
`Documentation <https://moderngl.readthedocs.io/>`_

`ModernGL on github <https://github.com/cprogrammer1994/ModernGL>`_

`Examples <https://github.com/cprogrammer1994/ModernGL/tree/master/examples>`_

OpenGL is a great environment for developing portable, platform independent,
interactive 2D and 3D graphics applications. The API implementation in Python
is cumbersome, resulting in applications with high latency. To solve this
problem we have developed ModernGL, a wrapper over OpenGL that simplifies the
creation of simple graphics applications like scientific simulations, small
games or user interfaces. Usually, acquiring in-depth knowledge of OpenGL
requires a steep learning curve. In contrast, ModernGL is easy to learn and
use, moreover it is capable of rendering with the same performance and quality,
with less code written.
'''

keywords = [
    'ModernGL',
    'OpenGL',
    'PyOpenGL',
    'rendering',
    'graphics',
    'shader',
    'GLSL',
    'GPU',
    'visualization',
    '2D',
    '3D',
]

classifiers = [
    'Development Status :: 5 - Production/Stable',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Topic :: Games/Entertainment',
    'Topic :: Multimedia :: Graphics',
    'Topic :: Multimedia :: Graphics :: 3D Rendering',
    'Topic :: Scientific/Engineering :: Visualization',
    'Programming Language :: Python :: 3 :: Only',
]

setup(
    name='moderngl',
    version='5.0.4',
    description=short_description,
    long_description=long_description.strip(),
    url='https://github.com/cprogrammer1994/ModernGL',
    author='Szabolcs Dombi',
    author_email='cprogrammer1994@gmail.com',
    license='MIT',
    classifiers=classifiers,
    keywords=keywords,
    packages=['moderngl'],
    install_requires=install_requires,
    ext_modules=[mgl],
    platforms=['any'],
)



import unittest

from common import get_context


class TestCase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        raise unittest.SkipTest('NYI')
        cls.ctx = get_context()

    def test_compute_shader(self):
        if self.ctx.version_code < 430:
            self.skipTest('OpenGL 4.3 is not supported')

        src = '''
            #version 430
            layout (local_size_x = 1, local_size_y = 1, local_size_z = 1) in;
            void main() {
            }
        '''

        compute_shader = self.ctx.compute_shader(src)
        self.assertEqual(compute_shader.source, src)


if __name__ == '__main__':
    unittest.main()



import struct
import unittest

import moderngl

from common import get_context


class TestCase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.ctx = get_context()

        cls.prog = cls.ctx.program(
            vertex_shader='''
                #version 330

                in int a_in;
                in int b_in;
                in int c_in;
                in int d_in;

                out int a_out;
                out int b_out;
                out int c_out;
                out int d_out;

                void main() {
                    a_out = a_in * 2;
                    b_out = b_in * 2;
                    c_out = c_in * 2;
                    d_out = d_in * 2;
                }
            ''',
            varyings=['a_out', 'b_out', 'c_out', 'd_out']
        )

    def test_padding_1(self):
        buf = self.ctx.buffer(struct.pack('=ixi12xii', 1, 2, 3, 4))
        res = self.ctx.buffer(reserve=16)

        vao = self.ctx.vertex_array(self.prog, [
            (buf, 'i x i 12x i i', 'a_in', 'b_in', 'c_in', 'd_in'),
        ])

        vao.transform(res, moderngl.POINTS)
        a, b, c, d = struct.unpack('=iiii', res.read())

        self.assertEqual(a, 2)
        self.assertEqual(b, 4)
        self.assertEqual(c, 6)
        self.assertEqual(d, 8)

    def test_padding_2(self):
        buf = self.ctx.buffer(struct.pack('=i8xi8xi8xi8x', 1, 2, 3, 4))
        res = self.ctx.buffer(reserve=64)

        vao = self.ctx.vertex_array(self.prog, [
            (buf, 'i 8x', 'a_in'),
        ])

        vao.transform(res, moderngl.POINTS)

        self.assertEqual(vao.vertices, 4)

        a1, a2, a3, a4 = struct.unpack('=i12xi12xi12xi12x', res.read())

        self.assertEqual(a1, 2)
        self.assertEqual(a2, 4)
        self.assertEqual(a3, 6)
        self.assertEqual(a4, 8)

    def test_padding_3(self):
        buf = self.ctx.buffer(struct.pack('=128xiiii', 1, 2, 3, 4))
        res = self.ctx.buffer(reserve=16)

        vao = self.ctx.vertex_array(self.prog, [
            (buf, '128x i i i i', 'a_in', 'b_in', 'c_in', 'd_in'),
        ])

        vao.transform(res, moderngl.POINTS)
        a, b, c, d = struct.unpack('=iiii', res.read())

        self.assertEqual(a, 2)
        self.assertEqual(b, 4)
        self.assertEqual(c, 6)
        self.assertEqual(d, 8)


if __name__ == '__main__':
    unittest.main()



import unittest

from common import get_context


class TestCase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.ctx = get_context()

    def test_framebuffer_read_channels(self):
        fbo = self.ctx.framebuffer([
            self.ctx.renderbuffer((4, 4)),
        ])

        fbo.clear(65 / 255, 66 / 255, 67 / 255, 68 / 255)

        self.assertEqual(fbo.read(components=1), b'A' * 16)
        self.assertEqual(fbo.read(components=2), b'AB' * 16)
        self.assertEqual(fbo.read(components=3), b'ABC' * 16)
        self.assertEqual(fbo.read(components=4), b'ABCD' * 16)


if __name__ == '__main__':
    unittest.main()
